import logging
import time

import cv2
import sys


class GSHLSPlayer:

    @classmethod
    async def create(cls, gsdbs, url, onframe):
        self = GSHLSPlayer()
        self.gsdbs = gsdbs
        self.url = url
        self.onframe = onframe
        self._logger = logging.getLogger(__name__)
        self.url += f"&vision=true&session={self.gsdbs.cookiejar.get('session')}&signature={self.gsdbs.cookiejar.get('signature')}"
        cap = cv2.VideoCapture(self.url)

        if cap.isOpened() == False:
            self._logger.error("unable to open playlist")
            return

        while True:
            ret, frame = cap.read()
            if not ret:
                break
            onframe(self.gsdbs, "hls_playlist", frame)
        cap.release()
        cv2.destroyAllWindows()


import ast
import json
import logging

import socketio
from aiortc import MediaStreamTrack, RTCPeerConnection, RTCIceServer, RTCConfiguration
from aiortc.contrib.media import MediaRelay, MediaBlackhole
from aiortc.rtcrtpreceiver import RemoteStreamTrack

relay = MediaRelay()


class VideoTransformTrack(MediaStreamTrack):
    kind = "video"

    def __init__(self, track, gsdbs, source, onframe):
        super().__init__()
        self.track = track
        self.onframe = onframe
        self.source = source
        self.gsdbs = gsdbs

    async def recv(self):
        frame = await self.track.recv()
        await self.onframe(self.gsdbs, self.source, frame)
        return frame


class GSPeerConnectionWatcher:

    @classmethod
    async def create(cls, gsdbs, target, onframe=None, onmessage=None, ontrack=None, ontrackended=None):
        self = GSPeerConnectionWatcher()
        self.rtconfiList = []
        self.sio = socketio.AsyncClient()
        self.gsdbs = gsdbs
        self.onframe = onframe
        self.target = target
        self.onmessage = onmessage
        self.ontrack = ontrack
        self.ontrackended = ontrackended
        self.logger = logging.getLogger(__name__)

        if self.gsdbs.credentials["stunenable"]:
            self.rtconfiList.append(RTCIceServer(self.gsdbs.credentials["stunserver"]))
        if self.gsdbs.credentials["turnenable"]:
            self.rtconfiList.append(RTCIceServer(self.gsdbs.credentials["turnserver"],
                                                 self.gsdbs.credentials["turnuser"],
                                                 self.gsdbs.credentials["turnpw"]))

        @self.sio.event
        async def connect():
            self.logger.info('connection established')

        @self.sio.event
        async def joined(id):
            self.logger.info("joined")

        @self.sio.event
        async def broadcaster(id):
            if len(self.rtconfiList) > 0:
                self.peerConnections = RTCPeerConnection(configuration=RTCConfiguration(self.rtconfiList))
            else:
                self.peerConnections = RTCPeerConnection()

            self.peerConnections.addTransceiver('video', direction='recvonly')

            @self.peerConnections.on("iceconnectionstatechange")
            async def on_iceconnectionstatechange():
                if self.peerConnections.iceConnectionState == "complete":
                    pass

                if self.peerConnections.iceConnectionState == "failed":
                    await self.peerConnections.close()

            @self.peerConnections.on("track")
            async def on_track(track):
                if track.kind == "video":
                    if self.ontrack is not None:
                        self.logger.info("on track received.Recording started")
                        await self.ontrack(self.gsdbs, track, self.target)
                    if self.onframe is not None:
                        self.logger.info("on track received. onframe started")
                        blackHole = MediaBlackhole()
                        mediaTrack = VideoTransformTrack(relay.subscribe(track), self.gsdbs, self.target, self.onframe)
                        blackHole.addTrack(mediaTrack)
                        await blackHole.start()

                # @track.on("ended")
                # async def on_ended():
                #     await self.ontrackended(self.gsdbs, self.target)

            await self.peerConnections.setLocalDescription(await self.peerConnections.createOffer())
            await self.sio.emit("watcher",
                                {"target": self.target,
                                 "sdp": {"type": self.peerConnections.localDescription.type,
                                         "sdp": self.peerConnections.localDescription.sdp}})

        @self.sio.event
        async def answer(id, description):
            if isinstance(description, dict):
                desc = type('new_dict', (object,), description)
            else:
                desc = type('new_dict', (object,), ast.literal_eval(description))
            await self.peerConnections.setRemoteDescription(desc)

        @self.sio.event
        async def disconnectBroadcaster(id, target):
            await self.ontrackended(self.gsdbs, target)

        if "localhost" in self.gsdbs.credentials["signalserver"]:
            connectURL = f'{self.gsdbs.credentials["signalserver"]}:{str(self.gsdbs.credentials["signalport"])}'
        else:
            connectURL = self.gsdbs.credentials["signalserver"]

        await self.sio.connect(
            f'{connectURL}?gssession={self.gsdbs.cookiejar.get("session")}.{self.gsdbs.cookiejar.get("signature")}{self.gsdbs.credentials["cnode"]}&target={self.target}')
        await self.sio.wait()



import asyncio
import json
import logging

import socketio


class GSCNodeFunctionReceiverSocket:
    @classmethod
    async def create(cls, sio, gsdbs, oncnodefunction):
        self = GSCNodeFunctionReceiverSocket()
        self.sio = sio
        self.gsdbs = gsdbs
        self.oncnodefunction = oncnodefunction
        self._logger = logging.getLogger(__name__)

        @self.sio.event
        async def connect():
            self._logger.info('oncnodefunction connected')

        @self.sio.event
        async def oncnodefunction(id, msg):
            self.oncnodefunction(self.gsdbs, id, msg)

        connectURL = ""

        if "localhost" in self.gsdbs.credentials["signalserver"]:
            connectURL = f'{self.gsdbs.credentials["signalserver"]}:{str(self.gsdbs.credentials["signalport"])}'
        else:
            connectURL = self.gsdbs.credentials["signalserver"]

        await self.sio.connect(
            f'{connectURL}?gssession={self.gsdbs.cookiejar.get("session")}.{self.gsdbs.cookiejar.get("signature")}{self.gsdbs.credentials["cnode"]}&global=true')
        await self.sio.wait()

    def sendcnodefunctionResult(self, id, msg):
        self.sio.emit("answer", id, msg)


class GSCNodeFunctionReceiver:
    def __init__(self, gsdbs, oncnodefunction):
        self.sio = socketio.AsyncClient()
        self.gsdbs = gsdbs
        self.oncnodefunction = oncnodefunction

    def startSocket(self):
        loop = asyncio.get_event_loop()
        loop.run_until_complete(GSCNodeFunctionReceiverSocket.create(self.sio, self.gsdbs, self.oncnodefunction))
        # asyncio.run(GSCNodeFunctionReceiverSocket.create(self.sio, self.gsdbs, self.oncnodefunction))

    async def sendcnodefunctionanswer(self, id, msg):
        await self.sio.emit("answer", {"id": id, "message": json.dumps(msg)})


class GSCNodeFunctionCallerSocket:
    @classmethod
    async def create(cls, sio, gsdbs, oncnodefunctionanswer):
        self = GSCNodeFunctionCallerSocket()
        self.sio = sio
        self.gsdbs = gsdbs
        self._logger = logging.getLogger(__name__)
        self.oncnodefunctionanswer = oncnodefunctionanswer

        @self.sio.event
        async def connect():
            self._logger.info('oncnodefunction connected')

        @self.sio.event
        async def answer(id, message):
            self.oncnodefunctionanswer(self.gsdbs, message)

        connectURL = ""

        if "localhost" in self.gsdbs.credentials["signalserver"]:
            connectURL = f'{self.gsdbs.credentials["signalserver"]}:{str(self.gsdbs.credentials["signalport"])}'
        else:
            connectURL = self.gsdbs.credentials["signalserver"]

        await self.sio.connect(
            f'{connectURL}?gssession={self.gsdbs.cookiejar.get("session")}.{self.gsdbs.cookiejar.get("signature")}{self.gsdbs.credentials["cnode"]}&caller=true')
        await self.sio.wait()


class GSCNodeFunctionCaller:
    def __init__(self, gsdbs, oncnodefunctionanswer):
        self.sio = socketio.AsyncClient()
        self.gsdbs = gsdbs
        self.oncnodefunctionanswer = oncnodefunctionanswer

    def startSocket(self):
        asyncio.run()
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            GSCNodeFunctionCallerSocket.create(self.sio, self.gsdbs, self.oncnodefunctionanswer))

    async def sendcnodefunction(self, target, msg):
        await self.sio.emit("oncnodefunction", {"target": target, "message": json.dumps(msg)})
